
require "import"
import "android.widget.*"
import "android.view.*"
import "android.text.InputType"
import "android.os.*"
import "android.speech.tts.TextToSpeech"

local activity = activity
local tts

function onCreate(savedInstanceState)
    activity.setTitle("Age Calculator")
    layout = LinearLayout(activity)
    layout.setOrientation(1)
    
    -- Date Inputs
    local dayInput = EditText(activity)
    dayInput.setHint("Enter Day")
    dayInput.setInputType(InputType.TYPE_CLASS_NUMBER)
    layout.addView(dayInput)

    local monthInput = EditText(activity)
    monthInput.setHint("Enter Month")
    monthInput.setInputType(InputType.TYPE_CLASS_NUMBER)
    layout.addView(monthInput)

    local yearInput = EditText(activity)
    yearInput.setHint("Enter Year")
    yearInput.setInputType(InputType.TYPE_CLASS_NUMBER)
    layout.addView(yearInput)
    
    -- Time Inputs
    local hourInput = EditText(activity)
    hourInput.setHint("Enter Hour")
    hourInput.setInputType(InputType.TYPE_CLASS_NUMBER)
    layout.addView(hourInput)

    local minuteInput = EditText(activity)
    minuteInput.setHint("Enter Minute")
    minuteInput.setInputType(InputType.TYPE_CLASS_NUMBER)
    layout.addView(minuteInput)

    -- Buttons
    local clearButton = Button(activity)
    clearButton.setText("Clear")
    layout.addView(clearButton)
    
    local calculateButton = Button(activity)
    calculateButton.setText("Calculate Age")
    layout.addView(calculateButton)
    
    -- Result Display
    local resultTextView = TextView(activity)
    resultTextView.setTextSize(18)
    layout.addView(resultTextView)
    
    calculateButton.setOnClickListener(View.OnClickListener {
        onClick = function(view)
            local day = tonumber(dayInput.getText().toString())
            local month = tonumber(monthInput.getText().toString())
            local year = tonumber(yearInput.getText().toString())
            local hour = tonumber(hourInput.getText().toString())
            local minute = tonumber(minuteInput.getText().toString())

            if day and month and year and hour and minute then
                local ageInfo = calculateAge(day, month, year, hour, minute)
                resultTextView.setText("Your age is: " .. ageInfo)

                speakOut("You are " .. ageInfo)
            else
                resultTextView.setText("Please enter a valid date and time")
                speakOut("Please enter a valid date and time")
            end
        end
    })
    
    clearButton.setOnClickListener(View.OnClickListener {
        onClick = function(view)
            dayInput.setText("")
            monthInput.setText("")
            yearInput.setText("")
            hourInput.setText("")
            minuteInput.setText("")
            resultTextView.setText("")
        end
    })
    
    activity.setContentView(layout)
    
    tts = TextToSpeech(activity, nil)
end

function calculateAge(day, month, year, hour, minute)
    local birthTime = os.time{year=year, month=month, day=day, hour=hour, min=minute}
    local currentTime = os.time()
    local diffTime = os.difftime(currentTime, birthTime)
    
    local years = math.floor(diffTime / (365.25 * 24 * 3600))
    local days = math.floor(diffTime / (24 * 3600)) % 365
    local months = math.floor(days / 30.44)
    local hours = math.floor(diffTime / 3600) % 24
    local minutes = math.floor(diffTime / 60) % 60

    local ageInfo = years .. " years, " .. months .. " months, " .. days .. " days, " .. hours .. " hours, " .. minutes .. " minutes"
    
    return ageInfo
end

function speakOut(text)
    tts.speak(text, TextToSpeech.QUEUE_FLUSH, nil, nil)
end

function onDestroy()
    if tts then
        tts.stop()
        tts.shutdown()
    end
end
